#include	<string.h>
#include	<stdlib.h>

#include	"column.hpp"
#include	"params.hpp"

/* ---------------------------------------------------------------------
	Set up the default mode, record length etc.
	Default is text mode with maximum line length of 255. Processing
	occurs for all records.
   --------------------------------------------------------------------- */

void	Params::defaults()
{
	fQuestion		=	FALSE;
	fTextMode		=	TRUE;
	iRecLen			=	255;
	iStartLine		=	0;
	iEndLine			=	0;
}

/* ---------------------------------------------------------------------
	Read the command line arguments to determine the record lengths,
	record number range and whether to use text mode.
	TransformSetup does the analysis of how to re-arrange the columns.
   --------------------------------------------------------------------- */

int	Params::Setup(int argc, char** argv)
{
	int	i;
	int	iNewRecLen, iMaxCol;
	int	cStart, cEnd, cLength, cNew;
	char	*parm;
	char	chType;

	/*	first determine the new record length after changes */

	iMaxCol = 0;
	iNewRecLen = 0;

	for (i=3; i<argc; i++) {
		parm = strupr(argv[i]);
		switch (*parm++) {
			case 'T'	:
				fTextMode = TRUE;
				iRecLen = read_param(&parm, iRecLen);
				break;

			case 'F'	:
				fTextMode = FALSE;
				iRecLen = read_param(&parm, iRecLen);
				break;

			case 'L'	:
				iStartLine	= read_param(&parm, iStartLine);
				iEndLine		= read_param(&parm, iEndLine);

				if (iEndLine - iStartLine < 0)
					error_handler(6, argv[i]);
				break;

			case 'R'	:
				cStart = read_param(&parm, 1);
				cEnd = read_param(&parm, iRecLen);

				if ((cLength = 1 + cEnd - cStart) <= 0)
					error_handler(6, argv[i]);

				chType = *parm++;
				cNew = read_param(&parm, 1);

				switch (chType) {
					case 'C'	:
						iNewRecLen += cLength;
						iMaxCol = max(iMaxCol, cNew + cLength - 1);
						break;

					case 'D'	:
						iNewRecLen -= cLength;
						break;

					case 'Z'	:
					case 'B'	:
					case 'S'	:
						iMaxCol = max(iMaxCol, cEnd);
						break;

					case 'M' :
						iMaxCol = max(iMaxCol, cNew + cLength - 1);
						break;

					default	:
						error_handler(5, argv[i]);
						break;
				}
				break;

			case 'Q'	:
				fQuestion = TRUE;
				break;

			default	:
				error_handler(5, argv[i]);
				break;
		}
	}

	if (fTextMode)
		iRecLen += 1;		/* adjust for end of string code */

	iNewRecLen += iRecLen;
	iNewRecLen = max(iNewRecLen, iMaxCol);

	if (iRecLen != iNewRecLen &&
		(iStartLine !=0 || iEndLine != 0) &&
		!fTextMode)
		error_handler(7, 0);

	return iNewRecLen;
}

/* ---------------------------------------------------------------------
	Check whether a line number is within a set of inclusive bounds.
	Note that if a bound is zero, it does not apply.
	The arguments are:
		pParams	=	specification of which line numbers are allowed.
		value		=	value to test
	Return value is TRUE if within range and FALSE otherwise.
   --------------------------------------------------------------------- */

BOOL	Params::in_range(int value)
{
	if (iStartLine	!= 0 && value < iStartLine)	return FALSE;
	if (iEndLine	!= 0 && value > iEndLine)		return FALSE;
	return TRUE;
}


